@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2009-2016 by Ilya Kotov                                 *
+ *   Copyright (C) 2009-2022 by Ilya Kotov                                 *
  *   forkotov02@ya.ru                                                      *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -30,12 +30,13 @@
 class QIODevice;
 class InputSource;
 class QmmpPluginCache;
+class EngineFactory;
 
 
 /*! @brief The AbstractEngine class provides the base interface class of audio audio engines.
  * @author Ilya Kotov <forkotov02@ya.ru>
  */
-class AbstractEngine : public QThread
+class QMMP_EXPORT AbstractEngine : public QThread
 {
  Q_OBJECT
 public:
@@ -43,7 +44,7 @@ public:
      * Object constructor.
      * @param parent Parent object.
      */
-    AbstractEngine(QObject *parent = 0);
+    AbstractEngine(QObject *parent = nullptr);
     /*!
      * Destructor.
      */
@@ -70,22 +71,13 @@ public:
      *  Pauses/resumes playback
      */
     virtual void pause() = 0;
-    /*!
-     * Mutes/Restores volume. Subclass should reimplement this function.
-     * @param muted - state of volume (\b true - mute, \b false - restore)
-     */
-    virtual void setMuted(bool muted) = 0;
-    /*!
-     * Returns mutex pointer.
-     */
-    QMutex *mutex();
     /*!
      * Creates Engine object.
      * @param s InputSource object.
      * @param parent Parent object.
      * Returns \b 0 if the given source is not supported.
      */
-    static AbstractEngine *create(InputSource *s, QObject *parent = 0);
+    static AbstractEngine *create(InputSource *s, QObject *parent = nullptr);
     /*!
      * Returns a list of engine factories.
      */
@@ -95,7 +87,15 @@ public:
      */
     static QList<EngineFactory*> enabledFactories();
     /*!
-     * Returns EngineFactory pointer which supports file \b path or 0 if file \b path is unsupported
+     * Returns a list of file name filters, i.e. "*.mp3 *.mpg".
+     */
+    static QStringList nameFilters();
+    /*!
+     * Returns a list of the supported mime types.
+     */
+    static QStringList contentTypes();
+    /*!
+     * Returns EngineFactory pointer which supports file \b path or 0 if file \b path is unsupported.
      */
     static EngineFactory *findByFilePath(const QString &path);
     /*!
@@ -103,23 +103,35 @@ public:
      * @param factory Engine plugin factory.
      * @param enable Plugin enable state (\b true - enable, \b false - disable)
      */
-    static void setEnabled(EngineFactory* factory, bool enable = true);
+    static void setEnabled(EngineFactory *factory, bool enable = true);
     /*!
      * Returns \b true if engine is enabled, otherwise returns \b false
+     * This function uses \b QObject::objectName() function.
      * @param factory Engine plugin factory.
      */
-    static bool isEnabled(EngineFactory* factory);
+    static bool isEnabled(const EngineFactory *factory);
+    /*!
+     * Returns \b true if engine is enabled, otherwise returns \b false
+     * @param engine Engine object.
+     */
+    static bool isEnabled(const AbstractEngine *engine);
     /*!
      * Returns plugin file path.
      * @param factory Engine plugin factory.
      */
-    static QString file(EngineFactory *factory);
+    static QString file(const EngineFactory *factory);
     /*!
      * Returns a list of supported protocols (including meta-protocols).
      * This fuction ignores disabled engines.
      */
     static QStringList protocols();
 
+protected:
+    /*!
+     * Returns mutex pointer.
+     */
+    QMutex *mutex();
+
 private:
     QMutex m_mutex;
     static void loadPlugins();
