@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2017 by Ilya Kotov                                 *
+ *   Copyright (C) 2008-2022 by Ilya Kotov                                 *
  *   forkotov02@ya.ru                                                      *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -25,6 +25,7 @@
 #include <QWidget>
 #include <QHash>
 #include <stddef.h>
+#include "qmmp_export.h"
 
 #define QMMP_VISUAL_NODE_SIZE 512 //samples
 
@@ -34,7 +35,7 @@ class VisualBuffer;
 /*! @brief The Visual class provides the base interface class of visualizations.
  *  @author Ilya Kotov <forkotov02@ya.ru>
  */
-class Visual : public QWidget
+class QMMP_EXPORT Visual : public QWidget
 {
     Q_OBJECT
 public:
@@ -43,7 +44,7 @@ public:
     * @param parent Parent object.
     * @param f Widget flags.
     */
-    Visual(QWidget *parent, Qt::WindowFlags f = 0);
+    Visual(QWidget *parent, Qt::WindowFlags f = Qt::WindowFlags());
     /*!
      * Destructor.
      */
@@ -56,18 +57,18 @@ public:
      * Returns plugin file path.
      * @param factory Visual plugin factory.
      */
-    static QString file(VisualFactory *factory);
+    static QString file(const VisualFactory *factory);
     /*!
      * Sets whether the visual plugin is enabled.
      * @param factory Visual plugin factory.
      * @param enable Plugin enable state (\b true - enable, \b false - disable)
      */
-    static void setEnabled(VisualFactory* factory, bool enable = true);
+    static void setEnabled(VisualFactory *factory, bool enable = true);
     /*!
      * Returns \b true if visual plugin is enabled, otherwise returns \b false
      * @param factory Effect plugin factory.
      */
-    static bool isEnabled(VisualFactory* factory);
+    static bool isEnabled(const VisualFactory *factory);
     /*!
      * Adds external visualization \b visual.
      */
@@ -82,7 +83,7 @@ public:
      * @param receiver Receiver object.
      * @param member A slot to receive changes of active visualizations list.
      */
-    static void initialize(QWidget *parent, QObject *receiver = 0, const char *member = 0);
+    static void initialize(QWidget *parent, QObject *receiver = nullptr, const char *member = nullptr);
     /*!
      * Returns a pointer to a list of created visual objects.
      */
@@ -109,13 +110,13 @@ public:
 
 public slots:
     /*!
-     * Starts visualization.
+     * Starts visualization. Default implementation does nothing.
      */
-    virtual void start() = 0;
+    virtual void start();
     /*!
-     * Stops visualization.
+     * Stops visualization. Default implementation does nothing.
      */
-    virtual void stop() = 0;
+    virtual void stop();
 
 signals:
     /*!
@@ -128,7 +129,7 @@ protected:
      * QWidget's close event. Reimplementation should call base function.
      * @param event QCloseEvent insatance.
      */
-    virtual void closeEvent (QCloseEvent *event);
+    virtual void closeEvent (QCloseEvent *event) override;
     /*!
      * Takes visualization data. Caller should allocate \b QMMP_VISUAL_NODE_SIZE
      * bytes for each channel. If buffer for right channel is not specified,
@@ -136,11 +137,12 @@ protected:
      * @param left Left channel buffer.
      * @param right Right channel buffer.
      */
-    bool takeData(float *left, float *right = 0);
+    bool takeData(float *left, float *right = nullptr);
 
 private:
+    static void createVisualization(VisualFactory *factory, QWidget *parent);
     static QList<VisualFactory*> *m_factories;
-    static QHash <VisualFactory*, QString> *m_files;
+    static QHash <const VisualFactory*, QString> *m_files;
     static void checkFactories();
     static QList<Visual*>  m_visuals;
     static QHash<VisualFactory*, Visual*> m_vis_map; //internal visualization
